local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )


return {
	name = STRINGS.IMPROVISATION.ACTIONS.SELFPARALYZE,
	getName = function( self, sim, unit )
		return STRINGS.IMPROVISATION.ACTIONS.SELFPARALYZE
	end,
	createToolTip = function( self, sim, abilityOwner )
		return abilityutil.formatToolTip(STRINGS.IMPROVISATION.ACTIONS.SELFPARALYZE, util.sformat(STRINGS.IMPROVISATION.ACTIONS.SELFPARALYZE_TIP,abilityOwner:getTraits().koTime) )
	end,

	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_paralyzerdose_small.png",

	alwaysShow = true,


	-- isTarget = function( self, sim, userUnit, targetUnit )
		-- local canUse = targetUnit:getAP() or targetUnit:getMP()
		-- and targetUnit:getPlayerOwner() == userUnit:getPlayerOwner()
		-- and simquery.isAgent(targetUnit)
		-- return canUse
	-- end,
	
	canUseAbility = function( self, sim, unit )

		-- Must have a user owner.
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end

		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,unit:getTraits().cooldown)
		end

		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end
		
		-- skill check
		return abilityutil.checkRequirements( unit, userUnit )
	end,
	
	executeAbility = function( self, sim, unit, userUnit, target )

		local userUnit = unit:getUnitOwner()
		local target = userUnit --sim:getUnit(target)	

		local x0,y0 = userUnit:getLocation()
		
		sim:dispatchEvent( simdefs.EV_UNIT_HEAL, { unit = userUnit, target = target, revive = false, facing = userUnit:getFacing() } )
		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.PARALYZED,x=x0,y=y0,color={r=1,g=1,b=0,a=1}} )

		target:setKO( sim, unit:getTraits().koTime )

		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit =target  } )

		if unit:getTraits().disposable then 
			inventory.trashItem( sim, userUnit, unit )
		else
			inventory.useItem( sim, userUnit, unit )
		end
		
		sim:triggerEvent( simdefs.TRG_UNIT_PARALYZED )
	end,
}