-- local array = include( "modules/array" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local cdefs = include( "client_defs" )
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )
local hilite_radius = include( "gameplay/hilite_radius" )

------------------------------------------------------------------

local emp_tooltip = class( abilityutil.hotkey_tooltip )

function emp_tooltip:init( hud, unit, abilityOwner, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
	self._targets = {}
	self._range = abilityOwner and abilityOwner:getTraits().range or 1
end

function emp_tooltip:activate( screen )
	abilityutil.hotkey_tooltip.activate( self, screen )

	local x0, y0 = self._unit:getLocation()
	local coords = {}

	local r = self._range
	for x1 = x0 - r, x0 + r do
	for y1 = y0 - r, y0 + r do
		table.insert(coords, x1)
		table.insert(coords, y1)
		local cell = self._unit._sim:getCell( x1, y1 )
		if cell then
			for i, u in pairs(cell.units) do
				if not self._targets[u]
				and simquery.canHear(u) and not u:isGhost()
				and u:getTraits().canKO and not u:isDown() then
					self._targets[u] = true
					self._game.boardRig:getUnitRig( u:getID() ):getProp():setRenderFilter( cdefs.RENDER_FILTERS["focus_target"] )
				end
			end
		end
	end
	end
	self._hiliteID = self._game.boardRig:hiliteCells( coords, {0.2,0.2,0.2,0.2} )
end

function emp_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
	self._game.boardRig:unhiliteCells( self._hiliteID )
	self._hiliteID = nil

    for u, _ in pairs(self._targets) do
        self._game.boardRig:getUnitRig( u:getID() ):refreshRenderFilter()
    end
	self._targets = {}
end    

------------------------------------------------------------------

return {
	name = STRINGS.IMPROVISATION.ACTIONS.SONICBOOM,
	
	getName = function( self, sim, unit, userUnit )
		return STRINGS.IMPROVISATION.ACTIONS.SONICBOOM
	end,
	createToolTip = function( self,sim,abilityOwner,abilityUser,targetID)
		return abilityutil.formatToolTip(STRINGS.IMPROVISATION.ACTIONS.SONICBOOM, STRINGS.IMPROVISATION.ACTIONS.SONICBOOM_TIP)
	end,
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
		return emp_tooltip( hud, abilityUser, abilityOwner, self, sim, abilityOwner, STRINGS.IMPROVISATION.ACTIONS.SONICBOOM_TIP )
	end,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
	alwaysShow = true,
	
	canUseAbility = function( self, sim, unit, userUnit )
		if not userUnit then
			return false
		end
		
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false,  util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown)
		end
		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end
		
		return abilityutil.checkRequirements( unit, userUnit ) and not userUnit:hasTrait("movingBody")
	end,
	
	executeAbility = function( self, sim, unit, userUnit )
		local x0, y0 = unit:getLocation()
		if x0 == nil then
			x0, y0 = userUnit:getLocation()
		end
		
		--make "visual" noise
		sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, range = 23 } )
		
		--make audible noise
		sim:emitSound({
			path = "SonicBoom/Grenades/bigflash_explo",
			range = 10, 
			-- hiliteCells = true,
			innocuous = true,
		}, x0, y0, unit )

		--knock out nearby receptible entities
		sim:startTrackerQueue(true)
		sim:startDaemonQueue()
		sim:dispatchEvent( simdefs.EV_KO_GROUP, true )
		
		local r = unit:getTraits().range or 1
		for x1 = x0 - r, x0 + r do
		for y1 = y0 - r, y0 + r do
			local cell = sim:getCell( x1, y1 )
			if cell then
				for i, u in pairs(cell.units) do
					if simquery.canHear(u) and not u:isGhost()
					and u:hasTrait("canKO") and not u:isDown() then
						log:write("SET KO ".. u:getUnitData().type)
						u:setKO( sim, 1 )
					end
				end
			end
		end
		end
		
		sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
		sim:startTrackerQueue(false)
		sim:processDaemonQueue()
		
		--alert all receptible entities
		local v = 3 --variance
		local offsets = {} --possible offsets
		for x1 = x0 - v, x0 + v do
		for y1 = y0 - v, y0 + v do
			local cell = sim:getCell( x1, y1 )
			if cell  then
				local imp
				for i,u in ipairs(cell.units) do
					-- check if there's anything making the cell impassable
					if not imp and u:hasTrait("dynamicImpass")
					and not u:hasTrait("emitterID") -- Guards assume this lase is on their side
					and not (u.isPC and u:isPC()) -- Guards may run into agents
					and not (u.isNPC and u:isNPC() and u:getBrain():getSituation().ClassType ~= simdefs.SITUATION_IDLE) -- Any non-stationary guards are ignored for this
					then
						imp = true
					end
				end
				if not imp then
					table.insert(offsets, {x1,y1})
				end
			end
		end
		end
		sim:forEachUnit(function(u)
			if simquery.canHear(u) then
				if u:hasTrait("isGuard") then
					local offset = offsets[ sim:nextRand(#offsets) ]
					u:getBrain():getSenses():addInterest(offset[1], offset[2], simdefs.SENSE_HEARING, simdefs.REASON_NOISE, userUnit)
					sim:processReactions(u)
				elseif u:getUnitData().type == "simsoundbug" then
					-- slight hack
					-- u._triggered = true
					-- u:getTraits().tracker_alert = true
					-- sim:trackerAdvance( 1, STRINGS.UI.ALARM_SOUND )	
					u:onTrigger(sim, simdefs.TRG_SOUND_EMITTED, {x=x0, y=y0, range=63, sourceUnit=userUnit})
					u:onTrigger(sim, simdefs.TRG_OVERWATCH)
				end
			end
		end)
		
		if unit:hasTrait("disposable") then 
			inventory.trashItem( sim, userUnit, unit )
		else
			inventory.useItem( sim, userUnit, unit )
		end
	end,
}