local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
-- local simdefs = include( "sim/simdefs" )

local _melee_ontooltip = commondefs.melee_template.onTooltip
local _item_ontooltip = commondefs.item_template.onTooltip

local tool_templates =
{
	item_sonicboom = util.extend(commondefs.item_template)
	{
		name = STRINGS.IMPROVISATION.SONICBOOM,
		desc = STRINGS.IMPROVISATION.SONICBOOM_TOOLTIP,
		flavor = STRINGS.IMPROVISATION.SONICBOOM_FLAVOR,
		icon = "itemrigs/FloorProp_emp.png",
		--profile_icon = "gui/items/icon-emp.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_sonicboom_small.png",	
		profile_icon_100 = "gui/icons/item_icons/icon-item_sonicboom.png",
		traits = { cooldown = 0, cooldownMax = 7, range = 1, },
		requirements = { hacking = 2 },
		abilities = { "carryable","recharge","sonicboom" },
		value = 700,
		floorWeight = 2, 
		ITEM_LIST = true,
	}, 
	item_knife1 = util.extend(commondefs.item_template)
	{
		name = STRINGS.IMPROVISATION.KNIFE1,
		desc = STRINGS.IMPROVISATION.KNIFE1_TOOLTIP,
		flavor = STRINGS.IMPROVISATION.KNIFE1_FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		--profile_icon = "gui/items/icon-paralyzer.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_knife1_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_knife1.png",	
		abilities = { "carryable", "inject_lethal" },
		requirements = {  },
		traits = {  },
		value = 200,
		floorWeight = 2,
		notSoldAfter = 24, 
		ITEM_LIST = true,
	},	
	item_knife2 = util.extend(commondefs.item_template)
	{
		name = STRINGS.IMPROVISATION.KNIFE2,
		desc = STRINGS.IMPROVISATION.KNIFE2_TOOLTIP,
		flavor = STRINGS.IMPROVISATION.KNIFE2_FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		--profile_icon = "gui/items/icon-paralyzer.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_knife2_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_knife2.png",	
		abilities = { "carryable", "recharge", "inject_lethal" },
		requirements = {  },
		traits = { armorPiercing = 1, armorPWRcost = 2, cooldownMax = 2, cooldown = 0, },
		value = 350,
		floorWeight = 2,
		ITEM_LIST = true,
		--blergh, add "armorPWRcost" and "armorPiercing"
		onTooltip = function( tooltip, unit, userUnit )
			_item_ontooltip(tooltip, unit, userUnit)
			local footer = tooltip._children[#tooltip._children]
			tooltip._children[#tooltip._children] = nil
			tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING, string.format(STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_DESC, unit:getTraits().armorPiercing), "gui/icons/arrow_small.png" )
			tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.ARMOR_PWR_COST, string.format(STRINGS.ITEMS.TOOLTIPS.ARMOR_PWR_COST_DESC, unit:getTraits().armorPWRcost), "gui/icons/arrow_small.png" )
			table.insert(tooltip._children, footer)
		end,
	},	
	item_epipen = util.extend(commondefs.item_template)
	{
		name = STRINGS.IMPROVISATION.EPIPEN,
		desc = STRINGS.IMPROVISATION.EPIPEN_TIP,
		flavor = STRINGS.IMPROVISATION.EPIPEN_FLAVOR,
		icon = "itemrigs/FloorProp_Bandages.png",
		--profile_icon = "gui/items/icon-paralyzer.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_medigel_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_med_gel.png",	
		abilities = { "carryable", "inject_lethal" },
		requirements = { anarchy = 2 },
		traits = { disposable = true },
		value = 200,
		floorWeight = 1,
		ITEM_LIST = true,
	}, 
	item_tagtazer = util.extend(commondefs.melee_template)
	{
		name = STRINGS.IMPROVISATION.TAGTAZER,
		desc = STRINGS.IMPROVISATION.TAGTAZER_TOOLTIP,
		flavor = STRINGS.IMPROVISATION.TAGTAZER_FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_tazer_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_tazer.png",		
		--profile_icon = "gui/items/icon-tazer-ftm.png",
		requirements = {  },
		traits = { damage = 2,  cooldown = 0, armorPiercing = 1, cooldownMax = 3, canTag= true, melee = true, tazer = true, level = 2 },
		value = 900,
		floorWeight = 3, --not actually used for weapons
		WEAPON_LIST = true,
		--blergh, add "canTag"
		onTooltip = function( tooltip, unit, userUnit )
			_melee_ontooltip(tooltip, unit, userUnit)
			local footer = tooltip._children[#tooltip._children]
			tooltip._children[#tooltip._children] = nil
			tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.CANTAG, string.format(STRINGS.ITEMS.TOOLTIPS.CANTAG_DESC), "gui/icons/arrow_small.png" )
			table.insert(tooltip._children, footer)
		end,
	},	
	item_holotov = util.extend(commondefs.grenade_template)
	{
        -- type = "stun_grenade",
		name = STRINGS.IMPROVISATION.HOLOTOV,
		desc = STRINGS.IMPROVISATION.HOLOTOV_TIP,
		flavor = STRINGS.IMPROVISATION.HOLOTOV_FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flash_grenade_small.png",	
		profile_icon_100 = "gui/icons/item_icons/icon-item_flash_grenade.png",		
		kanim = "kanim_flashgrenade",		
		sounds = {explode="SpySociety/Grenades/flashbang_explo", bounce="SpySociety/Grenades/bounce"},
		-- abilities = {  },
		-- requirements = {  },
		traits = { canSleep = true, range=3, explodes = 0 },
		value = 400,
		floorWeight = 1,
		locator=true,
		ITEM_LIST = true,
	}, 
}

return tool_templates