local function init( modApi )
    modApi.requirements = { "Sim Constructor" }

    local dataPath = modApi:getDataPath()
    KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
    KLEIResourceMgr.MountPackage( dataPath .. "/sound.kwad", "data" )
    MOAIFmodDesigner.loadFEV("SonicBoom.fev")

    modApi:addGenerationOption("sonicboom",  STRINGS.IMPROVISATION.OPTIONS.SONICBOOM , STRINGS.IMPROVISATION.OPTIONS.SONICBOOM_TIP, {noUpdate=true})
    modApi:addGenerationOption("epipen",  STRINGS.IMPROVISATION.OPTIONS.EPIPEN, STRINGS.IMPROVISATION.OPTIONS.EPIPEN_TIP, {noUpdate=true})
    modApi:addGenerationOption("tagtazer",  STRINGS.IMPROVISATION.OPTIONS.TAGTAZER, STRINGS.IMPROVISATION.OPTIONS.TAGTAZER_TIP, {noUpdate=true})
    -- modApi:addGenerationOption("holotov",  STRINGS.IMPROVISATION.OPTIONS.HOLOTOV, STRINGS.IMPROVISATION.OPTIONS.HOLOTOV_TIP, {noUpdate=true})
    modApi:addGenerationOption("selfparalyze",  STRINGS.IMPROVISATION.OPTIONS.SELFPARALYZE, STRINGS.IMPROVISATION.OPTIONS.SELFPARALYZE_TIP, {noUpdate=true})
end

local function lateInit( modApi )
    local abilitydefs = include( "sim/abilitydefs" )
    local simquery = include ( "sim/simquery" )
    local meleeAbility = abilitydefs._abilities[ "melee" ]
    local _melee_executeAbility = meleeAbility.executeAbility
    meleeAbility.executeAbility = function(self, sim, unit, userUnit, target )
		-- get this before attack, in case weapon breaks
		-- TODO refactor this whole thing as ability listening to melee event
		local weaponUnit = simquery.getEquippedMelee(userUnit)
		local canTag = weaponUnit and weaponUnit:getTraits().canTag
        _melee_executeAbility( self, sim, unit, userUnit, target )
        local targetUnit = sim:getUnit(target)
        if canTag and targetUnit
        and not targetUnit:getTraits().tagged and targetUnit:getBrain() then
            targetUnit:setTagged()        
        end
    end
end

local function unload()
    local guarddefs = include ("sim/unitdefs/guarddefs")
    for name, def in pairs(guarddefs) do
        if def.dropTable then
            -- log:write("drop table of ".. def.name)
            for i, drop in pairs(def.dropTable) do
                -- log:write(tostring(drop[1]))
                if drop[1] == "item_epipen" then
                    -- log:write("remove item_epipen from ".. def.name)
                    table.remove(def.dropTable, i)
                    -- break
                end
            end
        end
        if def.anarchyDropTable then
            for i, drop in pairs(def.anarchyDropTable) do
                if drop[1] == "item_epipen" then
                    table.remove(def.anarchyDropTable, i)
                    -- break
                end
            end
        end
    end
        
    local itemdefs = include ("sim/unitdefs/itemdefs")
    for name, def in pairs(itemdefs) do
        if def.abilities then
            -- log:write("abilities of ".. def.name)
            for i, ability in pairs(def.abilities) do
                -- log:write(ability)
                if ability == "selfparalyze" then
                    -- log:write("remove selfparalyze from ".. def.name)
                    table.remove(def.abilities, i)
                    -- break
                end
            end
        end
    end
end


local function load( modApi, options, params )
    --This seems dumber than it is. unload() resets all injections, which the game does not.
    --So in order to reset options, we can just call unload() before loading what we want. -M
    unload()

    local scriptPath = modApi:getScriptPath()
    local moditemdefs = include( scriptPath .. "/itemdefs" )

    if options.sonicboom and options.sonicboom.enabled then
        modApi:addItemDef( "item_sonicboom", moditemdefs.item_sonicboom )
        modApi:addAbilityDef( "sonicboom", scriptPath .."/abilities/sonicboom" )
    end
    if options.knives and options.knives.enabled then
        modApi:addItemDef( "item_knife1", moditemdefs.item_knife1 )
        modApi:addItemDef( "item_knife2", moditemdefs.item_knife2 )
        -- modApi:addAbilityDef( "cutthroat", scriptPath .."/cutthroat" )
    end
    if options.epipen and options.epipen.enabled then 
        modApi:addItemDef( "item_epipen", moditemdefs.item_epipen )
        modApi:addAbilityDef( "inject_lethal", scriptPath .."/abilities/inject_lethal" )

        -- chance to find this item on guards
        -- this needs unloading
        local guarddefs = include ("sim/unitdefs/guarddefs")
        for name, def in pairs(guarddefs) do
            if def.dropTable then
                for i, drop in pairs(def.dropTable) do
                    if drop[1] == "item_adrenaline" then
                        -- log:write("insert into ".. def.name .." with weight ".. tostring(drop[2]))
                        table.insert(def.dropTable, i+1, {"item_epipen", drop[2]})
                        break
                    end
                end
            end
            -- local dropTable = include("modules/weighted_list")( def.dropTable )
            -- for i = 1, #dropTable do
                -- log:write("INDEX ".. tostring(i) ..": ".. tostring(dropTable[i]))
            -- end
            if def.anarchyDropTable then
                for i, drop in pairs(def.anarchyDropTable) do
                    if drop[1] == "item_adrenaline" then
                        table.insert(def.anarchyDropTable, i+1, {"item_epipen", drop[2]})
                        break
                    end
                end
            end
        end
    end
    if options.tagtazer and options.tagtazer.enabled then
        modApi:addItemDef( "item_tagtazer", moditemdefs.item_tagtazer )
    end
    if options.holotov and options.holotov.enabled then
        modApi:addItemDef( "item_holotov", moditemdefs.item_holotov )
    end

    --the "not" is a backwards-compatibility, this change had no option at first -M
    if not options.selfparalyze or options.selfparalyze.enabled then
        -- this needs unloading
        modApi:addAbilityDef( "selfparalyze", scriptPath .."/abilities/selfparalyze" )
        local itemdefs = include ("sim/unitdefs/itemdefs")
        for name, def in pairs(itemdefs) do
            if def.abilities then
                for i, ability in pairs(def.abilities) do
                    if ability == "paralyze" then
                        table.insert(def.abilities, "selfparalyze")
                        break
                    end
                end
            end
        end
    end

end


local function initStrings(modApi)
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
    local MOD_STRINGS = include( scriptPath .. "/strings" )
    modApi:addStrings( dataPath, "IMPROVISATION", MOD_STRINGS)
end

return {
    initStrings = initStrings,
    init = init,
    lateInit = lateInit,
    load = load,
    unload = unload,
}